#!/usr/bin/env python
# encoding: utf-8

"""
SYNOPSIS

    prf2mat prf_file mat_file [rename_file]

DESCRIPTION

    Converts Nortek Paradopp format binary files to Matlab .mat files.
    
    Arguments
        prf_file -- Filename of the Paradopp file.
        mat_file -- Filename of Matlab file to write to
        rename_file -- Text file of variables to rename, split, and flatten.
        
    Extension currently supported
        .prf -- Aquadopp (and others?) files
        .vec -- Vector files
    

"""


import sys
import os
from pyparadopp import ParadoppFile


def parse_rename(filename):
    """
    Parse a rename file and return dict keys to rename, split and delete.
    
    This method parses the Rename file which which contains two whitespace
    delimited columns where the first column is the original dictionary key
    and the second column is the new name of the key.  There are also 3 keywords
    that can specifiy special actions on the files:
    
        split -- Split key by number of beams.  Split can also take a new name
            argument separated by a whitespace
        flatten -- Move all keys in dictionary up one level.  This is useful
            for the actual data values so they are at the base level of the
            dictionary
    
    Arguments
        filename -- File name to parse
    
    Returns
        dictionary with keys:
            rename -- Dictionary of old and new key names
            split -- List of keys to split
            flatten -- List of keys to flatten, ie, move one level up in
                the dictionary hierarchy.
    
    """

    class ParseError(Exception):
        """Class for rename parsing errors"""

        def __init__(self,msg):
            self.msg = msg

        def __str__(self):
            return msg

    with open(filename,'r') as fp:

        d = {'rename': {}, 'split': [], 'flatten': [] }
        
        for line_num, line in enumerate(fp):
            
            # Remove comments and continue if no data in line
            l = line.partition('#')[0].strip()
            if not l:
                continue
            fields = l.split()
            
            # Return old and new keys, along with an argument if its there
            # TODO: Make error more descriptive
            if len(fields) < 2:
                msg = 'Line {0}: Lines must contain at least two columns.'.format(line_num+1) # ones-based indexing for line numbers
                raise ParseError(msg)
            
            f0 = fields[0]
            f1 = fields[1]
            
            if len(fields) > 2:
                arg = fields[2]
            else:
                arg = False
            
            # Assign arguments to the structures
            if f0.lower() == 'delete':
                d['rename'][f1] = ''
            elif f0.lower() == 'flatten':
                d['flatten'].append(f1)
            elif f0.lower() == 'split':
                if arg:
                    d['rename'][f1] = arg
                    d['split'].append(arg)
                else:
                    d['split'].append(f1)
            else:
                d['rename'][f0] = f1

        return d


def main(argv):
    if len(argv) < 2:
        # Hardcode for testing
#        filename = 'WLP101.prf'
        filename = 'WLPV01.vec'
        matfile = 'test.mat'
        rfile = 'rename.txt'
    else:
        filename,matfile = argv[1:3]
        matfile = argv[2]

    if rfile:
        rdict = parse_rename(rfile)
    else:
        rdict = {}

    prf = ParadoppFile.from_file(filename, 1e4)
    prf.to_mat(matfile,rdict)

if __name__ == "__main__":
    main(sys.argv)

