# encoding: utf-8
"""
Base module for pyparadopp package
=====================================

    Classes
    
        DataCollection -- Class to collect data from Paradopp structure classes
            into a single class and set of data for optimization.
    
    Exceptions
    
        Error -- Base error class.
        ChecksumError -- Exception raised for mismatched checksums.
        SyncError -- Exception raised for mismatched sync bytes.
"""

# TODO: Check nBeams and nCells when combining datasets in DataCollection
# TODO: Enable time-limited based exporting
# TODO: Create Verbose version


import struct
import scipy.io as sio
import numpy as np

from .structs import *
from .utils import *
from progress_bar import ProgressBar

__all__ = ['Error', 'ChecksumError','SyncError', 'DataCollection']


class Error(Exception):
    """Base class for exceptions in this module."""
    pass


class ChecksumError(Error):
    """Exception raised for mismatched checksums
    
    Attributes:
        filename -- File where error occured
        offset -- File offset where error occured
    """
    
    def __init__(self, filename, offset):
        self.offset = offset
        self.filename = filename
    
    def __str__(self):
        return "Checksum mismatch in {filename} at 0x{offset:x}".format(filename=self.filename, offset=self.offset)


class SyncError(Error):
    """Exception raised for mismatched sync bytes
    
    Attributes:
        filename -- File where error occured
        offset -- File offset where error occured
        val -- value of sync byte
    """
    
    def __init__(self, filename, offset, val):
        self.val = val
        self.offset = offset
        self.filename = filename

    def __str__(self):
        # This is messy, but I think there's a way to simplfy it more pythonically
        return "Sync byte not equal to 0xa5 at 0x{offset:x} in {filename}. Value is: 0x{value:x}".format(value=self.val, 
                offset=self.offset, filename=self.filename)


class DataCollection(object):
    """Class to collect data structures into one for optimization"""
    
    def __init__(self, ndstruct, nRecs):
        self.type = ndstruct.name
        self.data = {}
        self.keys = ndstruct.data.keys()
        self.index = -1

        # Initialize any ndarrays in the collection
        for k in self.keys:
            
            # If we have an ndarray, initialize all the records
            if isinstance(ndstruct.data[k], np.ndarray):
                shape = (nRecs, ) + ndstruct.data[k].shape
                self.data[k] = np.empty(shape)
            
            # If we have a dict, create a new dictionary, so that we have a
            # dictionary of lists in the data
            elif type(ndstruct.data[k]) is dict:
                self.data[k] = {}
                for k2, v2 in ndstruct.data[k].iteritems():
                    if isinstance(ndstruct.data[k][k2], np.ndarray):
                        shape = (nRecs, ) + ndsruct.data[k][k2].shape
                        self.data[k][k2] = np.empty(shape)
                    else:
                        self.data[k][k2] = []
            
            # Other types get mapped to lists
            else:
                self.data[k] = []

        self.append(ndstruct)

    def append(self, ndstruct):
        """Append the new data structure to the Collection"""
        self.index +=1
        for k in self.keys:
            if isinstance(ndstruct.data[k], np.ndarray):
                self.data[k][self.index, :] = ndstruct.data[k]
            # If we have a dict, append the keys
            elif type(ndstruct.data[k]) is dict:
                for k2, v2 in ndstruct.data[k].iteritems():
                    if isinstance(ndstruct.data[k][k2], np.ndarray):
                        self.data[k][k2][self.index, :] = ndstruct.data[k][k2]
                    else:
                        self.data[k][k2].append(ndstruct.data[k][k2])
            
            else:
                self.data[k].append(ndstruct.data[k])