# encoding: utf-8
"""
ParadoppFile class
================================

   Base class for ParadoppFile sub-classes
"""

import os.path
import struct
import scipy.io as sio

from ..structs import *
from ..utils import *
from ..base import *

from ..progress_bar import ProgressBar

__all__ = ['ParadoppFile']


class ParadoppFile(object):
    """Class for Nortek Paradopp format binary files"""
    data = {}
    ext = None

    @classmethod
    def is_class_for_ext(cls,ext):
        return ext == cls.ext

    @classmethod
    def from_file(cls, filename, max_records=float("infinity")):
        """Factory to create ParadoppFile objects"""

        ext = os.path.splitext(filename)[1]
        for subclass in ParadoppFile.__subclasses__():
            if subclass.is_class_for_ext(ext):
                return subclass(filename, max_records)
        raise ValueError('Unknown file type for extension {0} .'.format(ext))

    class IterStructs(object):
        """Iterator for ParadoppFile data."""

        head_bytes = 4 # Number of bytes for the size, sync, id 
        head_size = 784 # Number of bytes in the header structs
        # Dictionary of sizes for structs w/o size data
        size_dict = {0x10: 24} 
        def __init__(self, filename, max_records=float("infinity"),
                     checksum=False,skip_head=False):
            self.filename = filename
            self.fp = open(filename, 'rb')
            self.max = max_records
            self.n = 0
            self.checksum = checksum

            if skip_head:
                self.fp.seek(self.head_size)

        def __iter__(self):
            return self

        def next(self):
            self.n+= 1
            if self.n > self.max:
                self.fp.close()
                raise StopIteration

            # First read the the size, sync, and id header
            # TODO: Should we issue warning if len(data) != size? 
            data = self.fp.read(self.head_bytes)

            # Quit if at end of file
            if not data or len(data) != self.head_bytes:
                self.fp.close()
                raise StopIteration

            # Assign size, id, sync and check sync
            sync, sid, size = struct.unpack("BBH", data)
            size = size*2  # Size is in words - words*2 => bytes

            # Some structs don't have size data, we fix that here.
            if sid in self.size_dict:
                size = self.size_dict[sid]

            if sync != 0xa5:
                offset = self.fp.tell()
                self.fp.close()
                raise SyncError(self.filename,offset,sync)

            # Assign data and check its length
            data = data + self.fp.read(size-self.head_bytes)

            # Quit if data != size 
            if len(data) != size:
                off = self.fp.tell()
                self.fp.close()
                raise StopIteration

            # Check checksum if requested
            if self.checksum and not checksum_test(data):
                offset = self.fp.tell()
                self.fp.close()
                raise ChecksumError(self.filename,offset)

            return (sid, size, data)

    def __init__(self, filename, max_records):

        print("-- Loading Paradopp File --")
        self.filename = filename
        self.max_records = max_records

        print('-- Getting Record Information --')
        self.record_info = self.get_record_info()

        print('-- Getting Header Information --')
        hw, head, user = self.get_headers()
        self.HWConfig = hw
        self.HeadConfig = head
        self.UserConfig = user
        
        # We could initialize an idDict here if necessary to speed things up

    def create_data(self,sid,data):
        """Convert string data into a Python class."""

        # Here we loop over the subclasses of the ParadoppDataStruct, looking
        # for one with a matching sid.  This way, we don't have to predefine
        # the sids anywhere else and the subclasses themselves hold their own
        # sids.  This method may be slow if loading lots of data, so we might
        # want to initialize a dictionary of ids in the __init__ of ParadoppFile

        for cls in ParadoppDataStruct.__subclasses__():
            if cls.is_class_for_id(sid):
                return cls(data)
        raise ValueError('Id: {0:x} is not a valid id'.format(sid))

    def get_headers(self):
        """Return the HWConfig, HeadConfig, and User config as classes."""

        headers = []
        for sid, size, data in self.IterStructs(self.filename,3):
            headers.append(self.create_data(sid,data))
        return tuple(headers)

    def get_record_info(self):
        """Return a dictionary of ids where the value is their number in the file."""
        info = {'sid': [], 'size': []}
        
        for sid, size, data in self.IterStructs(self.filename, self.max_records+3):
            info['sid'].append(sid)
            info['size'].append(size)
        info['total'] = len(info['sid'])
        return info

    def load_data(self):
        """Loads the data from the Paradopp file into memory"""

        nR = 0
        total_recs = min(self.record_info['total'],self.max_records)
        prog_bar = ProgressBar(0,total_recs,70)
        
        for sid, size, data in self.IterStructs(self.filename, self.max_records, True, True):
            nR += 1
            if nR % 100:
                prog_bar.updateAmount(nR)
                prog_bar.print_bar()

            # First convert the raw data into the appropriate class
            d = self.create_data(sid,data)
            
            # Append the data to the file, the subclasses handle this operation
            self.append(d)
            
        # Print newline at end of prog_bar
        print  

    def to_dict(self):
        """Returns all data as a dictionary"""

        d = {}
        for k in ('HWConfig','HeadConfig','UserConfig'):
            d[k] = getattr(self,k).data
        for k, v in self.data.items():
            d[k] = v.data
        return d

    def __str__(self):
        return str(self.to_dict())

    def to_mat(self, matfile, rdict={}):
        """ Saves data to a mat file, performing necessary conversions"""
        
        print('-- Loading Paradopp Data --')
        d = self.load_data()
        d = self.to_dict()
        
        d = walk_dict(d,convert_dates)
        
        if 'flatten' in rdict:
            d = flatten_keys(d, rdict['flatten'])
        if 'rename' in rdict:
            d = rename_keys(d, rdict['rename'])
        if 'split' in rdict:
            d = split_keys(d, rdict['split'])

        print('-- Saving to .mat file --')
        sio.savemat(matfile, d)
