# encoding: utf-8
"""
Aqudopp Specific Data Structures
================================

    Classes
    
        HRAquadoppProfileData -- Class for High-Resolution Aquadopp Profilier 
                         data structures.
"""

# TODO: What are VelLag, AmpLag, CorrLag?

import struct
import numpy as np
from datetime import timedelta

from ..utils import *
from .common import ParadoppDataStruct

__all__ = ['HRAquadoppProfileData']

class HRAquadoppProfileData(ParadoppDataStruct):
    """Class for Paradopp High Resolution Aquadopp Profilier Data"""
    name = "HRAquadoppProfileData"
    sid = 0x2a
    keys = "Time Milliseconds Error Battery SoundSpeed Heading Pitch Roll \
            PressureMSB PressureLSW Temperature AnaIn1 AnaIn2 Beams Cells \
            VelLag2 AmpLag2 CorrLag2 Vel Amp Corr"
    clean_keys = ['Milliseconds', 'PressureMSB', 'PressureLSW']
    scale_dict = {'Battery': 0.1, 'SoundSpeed': 0.1, 'Temperature': 0.01,
                  'Heading': 0.1, 'Pitch': 0.1, 'Roll': 0.1,
                  'Vel': 0.001, 'Corr': 0.01, 'Pressure': 0.001}


    # Unchecked keys: AnaIn1,AnaIn2 (should be ok, just don't have data in the file to check with)
    #    AmpLag2, CorrLag2, VelLag2 - Don't know what these values should look like
    #    Error - Again, not sure what it should be, but I get all zeros, which is presumably good
    #    Velocities look good, but need to check scaling as noted in the Sys. Int. Manual, pgs 31,44 
    def get_format(self, data):
        Beams, Cells = struct.unpack_from('BB', data, 34)
        base = "<4x6sHh2H3hBxHh2H2B6s3s3s6x"
        tail = '2x'
        if Cells%2==1:
            tail='x'+tail
        velocityFormat = '%ds' % (Beams*Cells*2)
        ampFormat = '%ds' % (Beams*Cells)
        format = base + velocityFormat + 2*ampFormat + tail
        return format

    def parse_data(self):
        
        # Assign reference to data, so we don't have to write self.data below
        data = self.data
        data['Time'] = bcddate2python(data['Time']) + timedelta(milliseconds=data['Milliseconds'])
        data['Pressure'] = data['PressureMSB']*65536.0 + data['PressureLSW']

        data['VelLag2'] = np.fromstring(data['VelLag2'], dtype=np.int16)
        data['AmpLag2'] = np.fromstring(data['AmpLag2'], dtype=np.uint8)
        data['CorrLag2'] = np.fromstring(data['CorrLag2'], dtype=np.uint8)        

        data['Vel'] = np.fromstring(data['Vel'], dtype=np.int16).reshape(data['Beams'], data['Cells']).transpose()
        data['Amp'] = np.fromstring(data['Amp'], dtype=np.uint8).reshape(data['Beams'], data['Cells']).transpose()
        data['Corr'] = np.fromstring(data['Corr'], dtype=np.uint8).reshape(data['Beams'], data['Cells']).transpose()
