# encoding: utf-8
"""
Paradopp Common Data Structures
===============================

    Classes
    
        ParadoppDataStruct -- Base class for all Paradopp Binary structures.
        HWConfig -- Hardware Configuration.
        HeadConfig -- Head Configuration.
        UserConfig -- User Configuration.
"""

# TODO: Create to_array method to simplify conversion of Vel, Amp, Corr to arrays
# TODO: Use s = Struct(format) to create a converter class (if needed)

import struct
import numpy as np
from ..utils import *

__all__ = ['ParadoppDataStruct', 'HWConfig', 'HeadConfig', 'UserConfig']


class ParadoppDataStruct(object):
    """Base Class for Paradopp binary data structures
    
    This class defines the common attributes and methods for the subclassed
    binary data structure classes to convert the binary data to a Python
    representation.  This class is not expected to be used directly, instead,
    the specific data structures use this class as a super class and are
    expected to comply with its interfaces.  The data from the original
    binary structure is stored in the attribute data as a dict after parsing on
    class initialization.  The subclass instances are created with a single
    argument - the binary data as read from the file.
    
    When a subclass instance is created, first the format of the binary
    structure is obtained via the get_format() method of the subclass.  By
    default get_format() returns the static format attribute.  Subclasses with
    variable format strings (like profilier data) must implement a get_format()
    method to generate the format string.  The dict keys are stored in a string
    attribute keys as space-separated values.  The parse_data() method is
    called to do special, per-structure type data parsing on the data.
    scale_data() is then called to scale the data by specified values and
    finally clean_data() deletes unneed data keys.
    
    Attributes
    
        clean_keys -- list of keys to remove after parsing.
        scale_dict -- Dictionary of scaling multipliers.  Each key represents a
            key in data to scale and it's value is the multiplier.
        format -- struct-style format of the data structure
        sid -- Data structure Id
    Methods
    
        get_format -- Return the struct-style format
        parse_data -- Parse the struct.unpack-ed data
        clean_data -- Remove keys from data
        scale_data -- Scale data values
    """
    
    clean_keys = []
    scale_dict = {}
    format = ""
    sid = None
    
    @classmethod
    def is_class_for_id(cls,sid):
        return sid == cls.sid
    
    def get_format(self, data):
        """Return struct-style format string for the data."""
        return self.format

    def parse_data(self, data):
        """Perform additional data parsing and return the data dict."""
        pass

    def clean_data(self):
        """Remove keys in data dict specified by clean_keys attribute."""
        for f in self.clean_keys:
            del self.data[f]

    def scale_data(self):
        """Scale keys in the data dict specified by the scale attribute."""
        for key, scale in self.scale_dict.items():
            self.data[key] = self.data[key]*scale

    def __init__(self, data):
        fmt = self.get_format(data)
        k = self.keys.split()
        v = struct.unpack(fmt, data)
        self.data = dict(zip(k, v))
        self.parse_data()
        self.scale_data()
        self.clean_data()

    def __str__(self):
        return str(self.data)


class HWConfig(ParadoppDataStruct):
    """Class for Paradopp Hardware Configuration Data"""
    name = "HWConfig"
    sid = 0x05
    keys = "SerialNo Config Frequency PICversion HWrevision RecSize Status \
            FWversion"
    format = "<4x14s6H12x4s2x"
    scale_dict = {'RecSize': 65536.0}


    # Unchecked keys: HWrevision, PICversion - these are ok if they are just unit16s
    def parse_data(self):
        
        # Assign reference to data, so we don't have to write self.data below
        data = self.data
        data['SerialNo']=data['SerialNo'].strip()
        
        # Config 
        c = data['Config']
        config = {'recorder_installed': bool(c & 0x01),
                  'compass_installed':  bool(c & 0x02) }
        data['Config'] = config
        
        # Status
        s = data['Status']
        status = {'vel_range': 'high' if (s & 0x01) else 'normal'}
        data['Status'] = status


class HeadConfig(ParadoppDataStruct):
    """Class for Paradopp Head Configuration Data"""
    name = "HeadConfig"
    keys = "Config Frequency Type SerialNo System NBeams"
    format = "<4x3H12s176s22xH2x"
    sid = 0x04
    
    # Unchecked keys: Type - no info on what this should be
    def parse_data(self):
        # Assign reference to data, so we don't have to write self.data below
        data = self.data
        
        data['SerialNo']=strip_null(data['SerialNo'])
        
        # Config
        c = data['Config']
        config = {'pressure_sensor': bool(c & 0x01),
                  'magnetometer_=sensor':  bool(c & 0x02),
                  'tilt_sensor': bool(c & 0x03),
                  'tilt_sensor_mount': 'down' if (c & 0x04) else 'up' }
        data['Config'] = config
        data.update(parse_system(data['System']))
        del(data['System'])


class UserConfig(ParadoppDataStruct):
    """Class for Paradopp User Configuration Data"""
    
    # TODO: Convert counts to distances
    name = "UserConfig"
    sid = 0x00
    keys = "T1 T2 T3 T4 T5 NPings AvgInterval NBeams TimCtrlReg PwrCtrlReg \
            CompassUpdRate CoordSystem NBins BinLength MeasInterval DeployName \
            WrapMode clockDeploy DiagInterval Mode AdjSoundSpeed NSampDiag \
            NBeamsCellDiag NPingsDiag ModeTest AnaInAddr SWVersion VelAdjTable \
            Comments WaveMode DynPercPos WaveT1 WaveT2 WaveT3 Nsamp AnaOutScale \
            CorrThresh TiLag2 QualConst"
    format = "<4x10H6x5H6sH6sI7H2s2x180s180s6H8x2H2xH30x16s2x"
    coord_sys = {0: 'ENU', 1: 'XYZ', 2: 'BEAM'}
    
     # Unchecked keys: QualConst,DynPerPos
    def parse_data(self):
        # Assign reference to data, so we don't have to write self.data below
        data = self.data
        
        data['clockDeploy'] = [bcddate2python(data['clockDeploy'])]
        data['DeployName'] = strip_null(data['DeployName'])
        data['Comments'] = strip_null(data['Comments'])
        data['CoordSystem'] = self.coord_sys[data['CoordSystem']]
        
        # TimCtrlReg
        t = data['TimCtrlReg']
        tcr = {'profile': 'continuous' if (t & 0x02) else 'single',
               'mode': 'continuous' if (t & 0x04) else 'burst',
               'power_level': ((t & 0x60) >> 4) + 1,
               'synchout_position': 'end' if (t & 0x80) else 'middle',
               'sample_on_synch': bool(t & 0x100),
               'start_on_synch': bool(t & 0x200)}
        data['TimCtrlReg'] = tcr

        # PwrCtrlReg
        p = data['PwrCtrlReg']
        pcr = {'power_level': ((p & 0x60) >> 4) + 1}
        data['PwrCtrlReg'] = pcr

        # Mode
        m = data['Mode']
        mode = {'user_soundspeed': bool(m & 0x01),
                'wavediag_mode': bool(m & 0x02),
                'analog_out_mode': bool(m & 0x04),
                'output_format': 'ADV' if (m & 0x08) else 'Vector',
                'scaling': 0.1   if (m & 0x10) else 1.0,
                'serial_output': bool(m & 0x20),
                'EasyQ': bool(m & 0x40),
                'stage': bool(m & 0x80),
                'output_power_for_ana_in': bool(m & 0x100)}
        data['Mode'] = mode

        # ModeTest
        m = data['ModeTest']
        mt = {'DSPfilter': bool(m & 0x01),
              'filter_output': 'correction' if (m & 0x02) else 'total'}
        data['ModeTest'] = mt

        # ModeWave
        m = data['WaveMode']
        wm = {'data_rate': (m & 0x01) + 1,
              'position': 'dynamic' if (m & 0x02) else 'fixed',
              'type': 'PctMeanPressure' if (m & 0x04) else 'PctMinRE'}
        data['wMode'] = wm