# encoding: utf-8
"""
Vector Specific Data Structures
================================

    Classes
    
        VectorHeader -- Class for Vector data header
        VectorVelocityData -- Class for Vector velocity data
        VectorSystemData -- Class for Vector System data
        VectorProbeCheck -- Class for Vector probe check data
"""

#import struct
import numpy as np
#from datetime import timedelta

from ..utils import *
from .common import ParadoppDataStruct

__all__ = ['VectorHeader', 'VectorVelocityData', 'VectorSystemData',
           'VectorProbeCheck']

class VectorHeader(ParadoppDataStruct):
    """Class for Vector Velocity Data Header"""
    name = "VectorVelocityDataHeader"
    keys = "Time NRecords Noise Correlation"
    sid = 0x12
    format = "<4x6sH4s4s22x"
    scale_dict = {'Correlation': 0.01}

    def parse_data(self):
        
        # Assign reference to data, so we don't have to write self.data below
        d = self.data
        d['Time'] = bcddate2python(d['Time'])
        d['Noise'] = np.fromstring(d['Noise'], dtype=np.uint16)
        d['Correlation'] = np.fromstring(d['Correlation'], dtype=np.uint16)


class VectorVelocityData(ParadoppDataStruct):
    """Class for Vector Velocity Data"""
    name = "VectorVelocityData"
    keys = "AnaIn2LSB Count PressureMSB AnaIn2MSB PressureLSW AnaIn1 Vel Amp Corr"
    sid = 0x10
    format = "<2x4B2H6s3s3s2x"
    clean_keys = ['AnaIn2LSB', 'AnaIn2MSB', 'PressureMSB', 'PressureLSW']
    scale_dict = {'Vel': 0.001, 'Corr': 0.01, 'Pressure': 0.001}
    
    def parse_data(self):
        # Not sure what to do with AnaIn2?
        # Assign reference to data, so we don't have to write self.data below
        d = self.data
        d['Pressure'] = d['PressureMSB']*65536.0 + d['PressureLSW']
        d['Vel'] = np.fromstring(d['Vel'], dtype=np.int16)
        d['Amp'] = np.fromstring(d['Amp'], dtype=np.uint8)
        d['Corr'] = np.fromstring(d['Corr'], dtype=np.uint8)


class VectorSystemData(ParadoppDataStruct):
    """Class for Vector System Data"""
    name = "VectorSystemData"
    keys = "Time Battery SoundSpeed Heading Pitch Roll Temperature Error Status AnaIn"
    format = "<4x6s2H4h2BH2x"
    sid = 0x11
    
    def parse_data(self):
        # Assign reference to data, so we don't have to write self.data below
        d = self.data
        d['Time'] = bcddate2python(d['Time'])
        s = d['Status']
        stat = {'orientation': 'down' if (0x01 & s) else 'up',
                'scaling': 0.1 if (0x02 & s) else 1.0,
                'pitch': bool(0x04 & s),
                'roll': bool(0x08 & s),
                'wakeup': (0x30 & s) >> 4,
                'power':  'Low' if (0x80 & s) else 'High'}
        d['Status'] = stat


class VectorProbeCheck(ParadoppDataStruct):
    """Class for Vector Probe Check data"""
    name = "VectorProbeCheck"
    keys = "NSamples SampleStart Amp"
    format = "<4x2H900s2x"
    sid = 0x07
    
    def parse_data(self):
        d = self.data
        d['Amp'] = np.fromstring(d['Amp'], dtype=np.uint8).reshape(3, 300).transpose()
