# encoding: utf-8
"""
Utility module for pyparadopp package
=====================================

    Methods
        convert_dates -- Converts potential datetime objects to Matlab datenum.
        checksum_test -- Test if checksums match the data.
        walk_dict -- Recursively walks a dictionary, apply a function.
        strip_null -- Strips null characters from end of string.
        bcddate2python -- Convert BCD style dates to datetime objects.
        bcd2dec -- Convert Binary Coded Decimal (BCD) to Decimal.
        parse_system -- Parse Head file System data
"""

# TODO: Alter strip null to strip everything after the first null char


import struct
import numpy as np
from datetime import datetime, timedelta
import matplotlib.dates as mpld

__all__ = ['convert_dates', 'checksum_test', 'walk_dict',
           'strip_null', 'bcddate2python', 'bcd2dec',
           'rename_keys', 'flatten_keys', 'split_keys', 'parse_system', 
           'dict_append']

def convert_dates(l):
    """
    Convert argument to Matlab datetime if it is datetime or a list of datetimes.
    """
    
    t = type(l)
    if t is datetime or (t is list and all(map(lambda x: type(x) is datetime,l))):
        l = mpld.date2num(l)
    return l


def checksum_test(data):
    """Test if the checksum matches the data"""

    # Checksums start with an initial value and are shorts, so the final sum 
    # has to be modded by a max_size to get the correct value.
    init_value = 0xb58c
    max_size = 0x10000

    vals = np.fromstring(data, dtype=np.uint16)
    checksum = vals[-1]
    val = (sum(vals[:-1]) + init_value) % max_size

    return val == checksum

def walk_dict(d,f):
    """
    Recursively walk dictionary applying function to non-dict key/value pairs
    """
    for k,v in d.items():
        if type(v) is dict:
            d[k] = walk_dict(v,f)
        else:
            d[k] = f(v)

    return d


def strip_null(s):
    """ Strip all values after the first null from string."""
    return s.partition('\x00')[0]


def dict_append(d1,d2):
    """Appends values in one dictionary to values in another"""
    d1keys = d1.keys()
    
    for k, v in d2.iteritems():
        if k in d1keys:
            if not d1[k] == list:
                d1[k] = [d1[k]]
            d1[k].append(d2[k])
        else:
            d1[k] = d2[k]
    return d1

def bcddate2python(s):
    """ Convert BCD style date to Python datetime."""
    s = struct.unpack('6B', s)
    vals = [bcd2dec(bcd) for bcd in s]
    if vals[4] >= 90:
        vals[4] += 1900
    else:
        vals[4] += 2000
    neword = [4, 5, 2, 3, 0, 1]
    return datetime(*[vals[i] for i in neword])


def bcd2dec(d):
    """Convert Binary Coded Decimal (BCD) to a decimal."""
    return d/16*10+d % 16


def parse_system(data):
    """Parse the System information and return a dict"""
    
    d = {}
    data = np.fromstring(data, dtype=np.int16)
    
    d['System5'] = data[0:4]
    d['TMatrix'] = data[4:13].reshape((3,3)) / 4096.0
    d['System7'] = data[13:21].reshape((2,4))
    #  Are there fill bytes at 42-43?
    d['System8'] = data[22:31].reshape((3,3))
    d['System9'] = data[31:40].reshape((3,3))
    d['System10'] = data[40:44]
    d['System11'] = data[44:48]
    d['System12'] = data[48:52]
    d['System13'] = data[52:56]
    d['System14'] = data[56:64].reshape(2,4)
    d['System15'] = data[64:80].reshape(4,4)
    d['System16'] = data[80:84]
    d['System17'] = data[84]
    d['System18'] = data[85]
    d['System19'] = data[86]
    d['System20'] = data[87]
    return d


def rename_keys(d, rename):
    """Recursively rename or delete dictionary keys"""

    rkeys = rename.keys()

    for k,v in d.items():

        # Recurse if needed
        if type(v) is dict:
            rename_keys(v, rename)
            
        # Reassign key if it's in the list.  Delete key if its value is empty
        if k in rkeys:
            new_key = rename[k]
            if new_key:
                d[new_key] = d[k]
            del(d[k])

    return d


def flatten_keys(d, flatten):
    """Flatten dictionary keys"""

    # Can we do this in an easy way recursively without worrying about side-effects?
    for k,v in d.items():
        if type(v) is dict and k in flatten:
            d.update(v)
            del(d[k])
    return d


def split_keys(d, split):
    """Recursively split dictionary keys"""
    
    for k, v in d.items():
        if type(v) is dict:
            split_keys(v, split)
        if k in split:
            N = d[k].shape[-1]
            for i in range(N):
                new_key = k + str(i+1)  # Add one for ones-based indexing
                d[new_key] = d[k][...,i]
            del(d[k])
    return d